#version 330 core
#define PI 3.141592653589793
#define y60 0.866025403784439

// Per vertex
in vec3 position;
in vec2 texCoords;
in vec3 normal;

// Per instance
in ivec3 blockPos;
in int blockTex;

out FragIn {
	vec2 texCoords;
	vec2 blockTex;//ivec3
	vec3 normal;
} fragIn;

uniform mat4 projMatrix;
uniform mat4 viewMatrix;
uniform float totalSize;
uniform vec3 cam;

void main() {
	float angleHalfHexagon = PI / totalSize;
	float radius = y60 / angleHalfHexagon;
	
	mat4 matrix = projMatrix * viewMatrix;
	
	vec4 pos = vec4(blockPos.x * 1.5 + position.x, blockPos.y + position.y, (blockPos.x + 2 * blockPos.z) + position.z / y60, 1.0);
	pos.z = mod(mod((pos.z + totalSize), totalSize*2) - cam.z / y60, totalSize*2) - totalSize;
	float mult = exp((pos.y - cam.y) / radius);
	float v = pos.z * angleHalfHexagon;
	float z = sin(v);
	float y = cos(v);
	
	fragIn.normal = vec3(normal.x, mat2(y, z, -z, y) * normal.yz);
	
	float scale = radius / sqrt(z*z+y*y);
	y *= scale;
	z *= scale;
	pos = vec4(pos.x - cam.x, y, z, 0) * mult;
	pos.y -= radius;
	pos.w = 1;
	gl_Position = matrix * pos; // temp
	fragIn.texCoords = texCoords;
	fragIn.blockTex = vec2(blockTex & 0xffff, (blockTex >> 16) & 0xffff);
}