package hexagon

import org.joml.Vector3f
import org.joml.Matrix4f
import hexagon.resource.Shader
import hexagon.world.coord.BlockRelWorld

object Camera {
	val unitX = new Vector3f(1, 0, 0).toImmutable()
	val unitY = new Vector3f(0, 1, 0).toImmutable()
  val unitZ = new Vector3f(0, 0, 1).toImmutable()
}

class Camera(var fov: Float, var aspect: Float, val near: Float, val far: Float) {
	val position = new Vector3f
	val rotation = new Vector3f
	var blockCoords: BlockRelWorld = null
	val placeInBlock = new Vector3f
	val projMatrix = new Matrix4f
	val viewMatrix = new Matrix4f
	val invProjMatr = new Matrix4f
	val invViewMatr = new Matrix4f
	
	val updateUniforms: Shader => Unit = s => {
		s.setUniformMat4("viewMatrix", viewMatrix)
		s.setUniform3f("cam", position.x, position.y, position.z)
	}
	val setProjMatrix: Shader => Unit = _.setUniformMat4("projMatrix", projMatrix)
	
	updateViewMatrix
	updateProjMatrix
	
	def setPosition(x: Float, y: Float, z: Float): Unit = {
		position.x = x
		position.y = y
		position.z = z
	}
	
	def move(x: Float, y: Float, z: Float): Unit = {
	  position.x += x
		position.y += y
		position.z += z
	}
	
	def setRotation(x: Float, y: Float, z: Float): Unit = {
	  rotation.x = x
		rotation.y = y
		rotation.z = z
	}
	
	def rotate(x: Float, y: Float, z: Float): Unit = {
	  rotation.x += x
		rotation.y += y
		rotation.z += z
	}
	
	def updateViewMatrix: Unit = {
		viewMatrix.identity()
		viewMatrix.rotate(rotation.z, Camera.unitZ)
		viewMatrix.rotate(rotation.x, Camera.unitX)
		viewMatrix.rotate(rotation.y, Camera.unitY)
		viewMatrix.invert(invViewMatr)
	}
	
	def updateProjMatrix: Unit = {
	  projMatrix.identity()
	  projMatrix.perspective(fov, aspect, near, far)
	  projMatrix.invert(invProjMatr)
	}
	
}