package hexagon.resource

import org.lwjgl.opengl.GL30
import org.lwjgl.opengl.GL15
import org.lwjgl.opengl.GL20
import org.lwjgl.opengl.GL11
import java.nio.FloatBuffer
import org.lwjgl.BufferUtils
import java.nio.IntBuffer
import scala.collection.mutable.ArrayBuffer

object Loader {
  def loadVAO = new VAOLoader
  
  val vaoToVbo = collection.mutable.Map.empty[Int, ArrayBuffer[Int]]
}

class VAOLoader {
  val vaoID = {
		val id = GL30.glGenVertexArrays()
		Loader.vaoToVbo.put(id, ArrayBuffer.empty)
		GL30.glBindVertexArray(id)
		id
	}
  
  def addVBO(attrNumber: Int, coordinateSize: Int, data: Array[Float]): VAOLoader = {
    val vboID = GL15.glGenBuffers()
		Loader.vaoToVbo.get(vaoID).get += vboID
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, vboID)
		GL15.glBufferData(GL15.GL_ARRAY_BUFFER, storeInFloatBuffer(data), GL15.GL_STATIC_DRAW)
		GL20.glVertexAttribPointer(attrNumber, coordinateSize, GL11.GL_FLOAT, false, 0, 0)
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, 0) // Unnecessary
		GL20.glEnableVertexAttribArray(attrNumber)
		this
  }
  
  def addIBO(ids: Array[Int]): VAOLoader = {
    val vbo = GL15.glGenBuffers()
		Loader.vaoToVbo.get(vaoID).get += vbo
		GL15.glBindBuffer(GL15.GL_ELEMENT_ARRAY_BUFFER, vbo)
		GL15.glBufferData(GL15.GL_ELEMENT_ARRAY_BUFFER, storeInIntBuffer(ids), GL15.GL_STATIC_DRAW)
		this
  }
  
  private def storeInFloatBuffer(data: Array[Float]): FloatBuffer = {
		val buffer = BufferUtils.createFloatBuffer(data.length)
		buffer.put(data)
		buffer.flip()
		buffer
	}
  
  private def storeInIntBuffer(data: Array[Int]): IntBuffer = {
		val buffer = BufferUtils.createIntBuffer(data.length)
		buffer.put(data)
		buffer.flip()
		buffer
	}
  
  // TODO: IBO and adding int-arrays
  
  def create = vaoID// TODO: implement some kind of useful class for this
}