package hexagon.world.render

import hexagon.world.storage.Chunk
import hexagon.renderer.VBO
import hexagon.renderer.VAOBuilder
import org.lwjgl.opengl.GL15
import hexagon.renderer.VAO
import hexagon.renderer.InstancedRenderer
import org.lwjgl.opengl.GL11
import hexagon.resource.Shader
import org.lwjgl.BufferUtils
import hexagon.world.storage.Block
import hexagon.world.coord.BlockRelChunk
import hexagon.world.coord.BlockRelWorld
import scala.collection.mutable.BitSet

class ChunkRenderer(chunk: Chunk) {
  class BlockRenderer(val side: Int) {
    val vao: VAO = new VAOBuilder(if (side < 2) 6 else 4, 1000)
      .addVBO(VBO(if (side < 2) 6 else 4, GL15.GL_STATIC_DRAW, 0).floats(0, 3).floats(1, 2).floats(2, 3).create().fillFloats(0, if (side < 2) setupBlockVBO(side) else setupBlockSideVBO(side)))
      .addVBO(VBO(1000, GL15.GL_DYNAMIC_DRAW, 1).ints(3, 3).ints(4, 1).create()).create()
    
    val renderer = new InstancedRenderer(vao, GL11.GL_TRIANGLE_STRIP)
    
    var instances = 0
    
    private def setupBlockVBO(s: Int): Seq[Float] = {
      val ints = Seq(1, 2, 0, 3, 5, 4)
      
      (0 until 6).map(i => {
    		val v = ints(if (s == 0) i else 5 - i) * Math.PI / 3
  			val x = Math.cos(v).toFloat
    		val z = Math.sin(v).toFloat
  			Seq(x, 1 - s, z,
  			    (1 + x) / 2, (1 + z) / 2,
  			    0, 1 - 2 * s, 0)
      }).flatten
    }
    
    private def setupBlockSideVBO(s: Int): Seq[Float] = {
      val nv = ((s - 1) % 6 - 0.5) * Math.PI / 3
      val nx = Math.cos(nv).toFloat
    	val nz = Math.sin(nv).toFloat
      
    	(0 until 4).map(i => {
    		val v = (s - 1 - i % 2) % 6 * Math.PI / 3
    		val x = Math.cos(v).toFloat
    		val z = Math.sin(v).toFloat
    		Seq(x, 1 - i / 2, z,
    		    1 - i % 2, i / 2,
    		    nx, 0, nz)
      }).flatten
    }
  }
  
  val blockRenderers = for (s <- 0 until 8) yield new BlockRenderer(s)
  
  val blockShader = Shader.getShader("block")
  
  def updateContent(): Unit = {
    val blocks = chunk.blocks.allBlocks
    val sidesToRender = Seq.fill(8)(new BitSet(16*16*16))
    for (b <- blocks) {
      val c = b.coord.getBlockRelChunk
      for (s <- Block.neighbors.indices) {
        val (i, j, k) = Block.neighbors(s)
        val (i2, j2, k2) = (c.cx + i, c.cy + j, c.cz + k)
        if ((i2 & ~15 | j2 & ~15 | k2 & ~15) == 0) {
        	val c2 = BlockRelChunk(i2, j2, k2)
          if (chunk.blocks.blockType(c2) == 0) {
            sidesToRender(s)(c.value) = true
          }
        } else if (chunk.world.getBlock(BlockRelWorld(chunk.coords.X * 16 + i2, chunk.coords.Y * 16 + j2, chunk.coords.Z * 16 + k2, chunk.world)).filter(_.blockType != 0).isEmpty) {
          sidesToRender(s)(c.value) = true
        }
      }
    }
		for (r <- blockRenderers) {
		  r.instances = 0
      val buf = BufferUtils.createByteBuffer(blocks.size * blockRenderers(2).vao.vbos(1).stride)
  	  for (block <- blocks) {
  	    if (sidesToRender(r.side)(block.coord.getBlockRelChunk.value)) {
    		  r.instances += 1
    		  Seq(block.coord.x, block.coord.y, block.coord.z, 0).foreach(buf.putInt)
  	    }
  	  }
      buf.flip()
      r.vao.vbos(1).fill(0, buf)
		}
  }
  
  def render(): Unit = {
    for (r <- blockRenderers) {
      if (r.side == 0 || r.side == 5 || true) {
        blockShader.setUniform1i("side", r.side)
        r.renderer.render(r.instances)
      }
    }
  }
}