package hexagon.scene

import org.joml.Vector3d
import org.joml.Vector3f
import org.lwjgl.glfw.GLFW._

import hexagon.HexBox
import hexagon.Main
import hexagon.world.coord.BlockCoord
import hexagon.world.storage.World
import hexagon.world.Player

class PlayerInputHandler(val player: Player) {
  var moveWithMouse = false

  def tick(): Unit = {
    updatePlayer
  }
  // TODO: make Map[key: Int, state: Int] so that the game only receives key presses when it's not overlayed, or make this method not always be called
  private def updatePlayer(): Unit = {
    val speed = if (glfwGetKey(Main.window, GLFW_KEY_LEFT_CONTROL) == GLFW_PRESS) {
      0.075f
    } else if (glfwGetKey(Main.window, GLFW_KEY_LEFT_ALT) == GLFW_PRESS) {
      12f
    } else if (glfwGetKey(Main.window, GLFW_KEY_RIGHT_CONTROL) == GLFW_PRESS) {
      120f
    } else {
      3f
    }
    val rSpeed = 0.05f
    if (player.flying) player.velocity.y = 0
    if (glfwGetKey(Main.window, GLFW_KEY_W) == GLFW_PRESS) {
      player.velocity.z -= Math.cos(player.rotation.y) * speed
      player.velocity.x += Math.sin(player.rotation.y) * speed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_S) == GLFW_PRESS) {
      player.velocity.z += Math.cos(player.rotation.y) * speed
      player.velocity.x -= Math.sin(player.rotation.y) * speed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_D) == GLFW_PRESS) {
      player.velocity.x += Math.cos(player.rotation.y) * speed
      player.velocity.z += Math.sin(player.rotation.y) * speed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_A) == GLFW_PRESS) {
      player.velocity.x -= Math.cos(player.rotation.y) * speed
      player.velocity.z -= Math.sin(player.rotation.y) * speed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_SPACE) == GLFW_PRESS) {
      if (player.flying) {
        player.velocity.y = speed
      } else if (player.velocity.y == 0) {
        player.velocity.y = 5f
      }
    }
    if (glfwGetKey(Main.window, GLFW_KEY_LEFT_SHIFT) == GLFW_PRESS) {
      if (player.flying) player.velocity.y = -speed
    }

    if (glfwGetKey(Main.window, GLFW_KEY_UP) == GLFW_PRESS) {
      player.rotation.x -= rSpeed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_DOWN) == GLFW_PRESS) {
      player.rotation.x += rSpeed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_LEFT) == GLFW_PRESS) {
      player.rotation.y -= rSpeed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_RIGHT) == GLFW_PRESS) {
      player.rotation.y += rSpeed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_PAGE_UP) == GLFW_PRESS) {
      player.rotation.z -= rSpeed
    }
    if (glfwGetKey(Main.window, GLFW_KEY_PAGE_DOWN) == GLFW_PRESS) {
      player.rotation.z += rSpeed
    }

    if (moveWithMouse) {
      val mouseMoved = Main.mouseMoved
      player.rotation.y += mouseMoved.x.toFloat * rSpeed * 0.1f
      player.rotation.x -= mouseMoved.y.toFloat * rSpeed * 0.1f
    }

    if (player.rotation.x < -math.Pi / 2) {
      player.rotation.x = -(math.Pi / 2).toFloat
    } else if (player.rotation.x > math.Pi / 2) {
      player.rotation.x = (math.Pi / 2).toFloat
    }

    if (player.rotation.y < 0) {
      player.rotation.y += (math.Pi * 2).toFloat
    } else if (player.rotation.y > math.Pi * 2) {
      player.rotation.y -= (math.Pi * 2).toFloat
    }

    if (player.rotation.z < 0) {
      player.rotation.z += (math.Pi * 2).toFloat
    } else if (player.rotation.z > math.Pi * 2) {
      player.rotation.z -= (math.Pi * 2).toFloat
    }

    player.velocity.x *= 0.75f
    player.velocity.z *= 0.75f

    val velLen = math.hypot(player.velocity.x, player.velocity.z).toFloat
    if (velLen > speed) {
      player.velocity.x *= speed / velLen
      player.velocity.z *= speed / velLen
    }
    
    if (!player.flying) {
      player.velocity.y -= 9.82f / 60
      player.velocity.div(60)
      val (pos, vel) = player.bounds.afterCollision(player.position, player.velocity, player.world)
//      println(vel)
//      println("_________________________________________")
      player.position.set(pos)
      player.velocity.set(vel)
      player.velocity.mul(60)
    } else {
      player.position.add(player.velocity.x / 60, player.velocity.y / 60, player.velocity.z / 60)
    }
  }
}
