package hexagon

import java.io.File
import org.lwjgl.system.Configuration
import org.lwjgl.glfw._
import org.lwjgl.glfw.GLFW._
import org.lwjgl.glfw.Callbacks._
import org.lwjgl.opengl._
import org.lwjgl.system.MemoryStack
import org.joml.Vector2i
import org.lwjgl.system.MemoryStack._
import scala.collection.mutable.ArrayBuffer
import hexagon.resource.Shader
import hexagon.resource.Resource
import org.joml.Vector2d
import org.lwjgl.system.MemoryUtil

object Main {
  {
    var file = new File("lib/natives")
	  if (!file.exists) file = new File("lib/natives-" + (if (System.getProperty("os.name").toLowerCase.contains("win")) "windows" else "linux"))
	  Configuration.LIBRARY_PATH.set(file.getAbsolutePath)
  }
	
	val windowSize = new Vector2i(920, 540)
	
  val window = initWindow()
  
  private val _mousePos = new Vector2d()
	private val _mouseMoved = new Vector2d()
	
	private val xposArr = MemoryUtil.memAllocDouble(1)
	private val yposArr = MemoryUtil.memAllocDouble(1)
	def mousePos = _mousePos.toImmutable()
	def mouseMoved = _mouseMoved.toImmutable()
  
  private val sceneList = ArrayBuffer.empty[Scene]
  def pushScene(scene: Scene): Unit = {
	  if (scene != null) {
	    sceneList += scene
	  }
	}
	def popScene(): Unit = {
	  if (sceneList.size > 0) {
	    val index = sceneList.size - 1
	    sceneList(index).unload()
	    sceneList.remove(index)
	  }
	}
  
  def loop(): Unit = {
    var prevTime = System.nanoTime
    var currentTime, realPrevTime = prevTime
    var realDeltaTime = 0L
		var (delta, ticks, frames, fps) = (0, 0, 0, 0)
		
		while (!glfwWindowShouldClose(window)) {
		  currentTime = System.nanoTime
			delta = ((currentTime - prevTime) * 1e-9 * 60).toInt
			realPrevTime = currentTime
			for (i <- 0 until delta) {
//				if (ticks % 10 == 0) world.updateLoadedChunks(camera)
				tick()
				ticks += 1
				if (ticks % 60 == 0) {
					fps = frames
					frames = 0
				}
				prevTime += 1e9.toLong / 60
			}
			//long tempTime = System.nanoTime - realPrevTime
			GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT)
			render()
			val error = GL11.glGetError
			if (error != GL11.GL_NO_ERROR) System.out.println("OpenGL error: " + error)
			
//			GL11.glFinish()
			
			frames += 1
			realDeltaTime = System.nanoTime - realPrevTime//tempTime
			
			glfwSwapBuffers(window) // swap the color buffers
			
			val debugInfo = ""//"    P: (" + camera.position.x + ", " + camera.position.y + ", " + camera.position.z + ")    C: (" + ((int) camera.blockCoords.x >> 4) + ", " + ((int) camera.blockCoords.y >> 4) + ", " + ((int) camera.blockCoords.z >> 4) + ")    R: (" + (float) (camera.rotation.x * 180 / Math.PI) + ", " + (float) (camera.rotation.y * 180 / Math.PI) + ", " + (float) (camera.rotation.z * 180 / Math.PI) + ")    v=" + (int) (100 * World.viewDistance) / 100f + (sides != 0xff ? "    s=" + Integer.toBinaryString(sides) : "")
			glfwSetWindowTitle(window, "Hexagon   " + fps + " fps   ms: " + (if ((realDeltaTime * 1e-6).toInt < 10) "0" else "") + (realDeltaTime * 1e-6).toInt + debugInfo)
			
			// Poll for window events. The key callback will only be
			// invoked during this call.
			glfwPollEvents
		}
		
//		world.saveWorld
  }
	
	private def render(): Unit = {
	  sceneList.foreach(_.render())
	}
	
	private def tick(): Unit = {
	  glfwGetCursorPos(window, xposArr, yposArr);
	  val oldMouseX = _mousePos.x
	  val oldMouseY = _mousePos.y
		_mousePos.set(xposArr.get(0), windowSize.y - yposArr.get(0));
		_mouseMoved.set(_mousePos.x - oldMouseX, _mousePos.y - oldMouseY)
	  
	  sceneList.foreach(_.tick())// TODO: should maybe be reversed
	}
  
  def run(): Unit = {
    initGL()
    Shader.init()
    pushScene(new MainMenuScene)
    pushScene(new GameScene)
		loop()
		
		destroy()
		
		// Free the window callbacks and destroy the window
		glfwFreeCallbacks(window)
		glfwDestroyWindow(window)
		
		// Terminate GLFW and free the error callback
		glfwTerminate()
		glfwSetErrorCallback(null).free()
  }
  
  private def processKeys(window: Long, key: Int, scancode: Int, action: Int, mods: Int): Unit = {//action: 0 = release, 1 = press, 2 = repeat
		sceneList.reverseIterator.forall(!_.processKeys(key, scancode, action, mods))
	}
  
  private def resizeWindow(width: Int, height: Int): Unit = {
    if (width != windowSize.x || height != windowSize.y) {
			GL11.glViewport(0, 0, width, height)
  		
  		sceneList.foreach(_.windowResized(width, height))
		}
		windowSize.set(width, height)
  }
  
  private def initWindow(): Long = {
    // Setup an error callback. The default implementation
		// will print the error message in System.err.
		GLFWErrorCallback.createPrint(System.err).set
		
		// Initialize GLFW. Most GLFW functions will not work before doing this.
		if (!glfwInit) throw new IllegalStateException("Unable to initialize GLFW")
		
		// Configure GLFW
		glfwDefaultWindowHints // optional, the current window hints are already the default
		glfwWindowHint(GLFW_VISIBLE, GLFW_FALSE) // the window will stay hidden after creation
		glfwWindowHint(GLFW_RESIZABLE, GLFW_TRUE) // the window will be resizable
		glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3)
		glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3)
		glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL11.GL_TRUE)
		glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE)
		glfwWindowHint(GLFW_SAMPLES, 1)
		
		// Create the window
		val window = glfwCreateWindow(windowSize.x, windowSize.y, "Hexagon", 0, 0)
		if (window == 0) throw new RuntimeException("Failed to create the GLFW window")
		
		// Setup a key callback. It will be called every time a key is pressed, repeated or released.
		glfwSetKeyCallback(window, processKeys)
		
		glfwSetWindowSizeCallback(window, (window, width, height) => {
			resizeWindow(width, height)
		})
		
		// Get the thread stack and push a new frame
		{
		  val stack = stackPush()
			val pWidth = stack.mallocInt(1) // int*
			val pHeight = stack.mallocInt(1) // int*
			
			// Get the window size passed to glfwCreateWindow
			glfwGetWindowSize(window, pWidth, pHeight)
			
			// Get the resolution of the primary monitor
			val vidmode = glfwGetVideoMode(glfwGetPrimaryMonitor)
			
			// Center the window
			glfwSetWindowPos(window, (vidmode.width - pWidth.get(0)) / 2, (vidmode.height - pHeight.get(0)) / 2)
			stack.pop()
		} // the stack frame is popped automatically
		
		// Make the OpenGL context current
		glfwMakeContextCurrent(window)
		// Enable v-sync
		glfwSwapInterval(1)
		
		// Make the window visible
		glfwShowWindow(window)
		window
  }
  
  private def initGL(): Unit = {
		GL.createCapabilities()
		
//		Display.create(new PixelFormat.withSamples(2).withDepthBits(24), new ContextAttribs(3, 3))//samples = 2 or 4
		
//		GL11.glClearColor(0.1f, 0.2f, 0.7f, 1)
//		GL11.glEnable(GL13.GL_MULTISAMPLE)
		GL11.glEnable(GL11.GL_DEPTH_TEST)
		GL11.glDepthFunc(GL11.GL_LEQUAL)
		
//		System.out.println(GL11.glGetInteger(GL12.GL_MAX_ELEMENTS_VERTICES))
  }
  
  def destroy(): Unit = {
    while (sceneList.size > 0) popScene()
    
    Resource.freeAllResources()
  }
  
  def main(args: Array[String]): Unit = {
		run()
  }
}