package hexagon.world.storage

import hexagon.world.coord.ColumnRelWorld
import hexagon.world.coord.BlockRelWorld
import hexagon.world.coord.ChunkRelWorld
import hexagon.world.coord.CoordUtils

class World {
  /* blockStorage
   * other world contents
   * methods for world interaction
   * method for loading/unloading blockColumns
   * block methods that will be forwarded to the correct chunk
   * worldGen references and methods
   * might rename this class to WorldStorage, and make another class World
  */
  
  /** Max-value: 20 */
  val worldSize = 5
  val ringSize = 1 << worldSize
  val ringSizeMask = ringSize - 1
  val totalSize = 16 * ringSize
  val totalSizeMask = totalSize - 1
  val halfHexAngle = (2 * math.Pi) / totalSize
  val radius = CoordUtils.y60 / halfHexAngle
  
  val columns = scala.collection.mutable.Map.empty[Long, ChunkColumn]
  for (i <- -8 until 8; j <- -8 until 8) columns += ColumnRelWorld(i, j, this).value -> new ChunkColumn(ColumnRelWorld(i, j, this), this)// TODO: temp
  
  def getColumn(coords: ColumnRelWorld): Option[ChunkColumn] = columns.get(coords.value)
  
  def getChunk(coords: ChunkRelWorld): Option[Chunk] = getColumn(coords.getColumnRelWorld) match {
    case Some(col) => col.getChunk(coords.getChunkRelColumn)
    case None => None
  }
  
  def getBlock(coords: BlockRelWorld): Option[Block] = getColumn(coords.getColumnRelWorld) match {
    case Some(col) => col.getBlock(coords.getBlockRelColumn)
    case None => None
  }
  
  def tick(): Unit = {
    columns.values.foreach(_.tick)
  }
  
}