#define y60 0.866025403784439

in FragIn {
	vec2 texCoords;
	flat int blockTex;
	vec3 normal;
} fragIn;

out vec4 color;

uniform sampler2DArray texSampler;
uniform float glow = 1.0;
uniform int side;
uniform int blocksPerRow;
uniform int texSize = 32;
uniform vec3 sun;

void main() {
	int texDepth = fragIn.blockTex & 0xffff;
	
#if isSide
	color = texture(texSampler, vec3(fragIn.texCoords, texDepth));
#else
	float yy = (fragIn.texCoords.y * 2 - 1) / y60;
	float xx = fragIn.texCoords.x + yy * 0.25;
	yy = (yy + 1) * 0.5;
	float zz = yy - xx + 0.5;
	vec3 pp = vec3(xx, yy, zz) * 2 - 1;
	vec3 cc = 1 - abs(pp);
	int ss = 4;
	if (pp.x >= 0) {
		if (pp.z < 0) {
			if (pp.y >= 0) {
				cc = cc.zxy;
				ss = 0;
			} else {
				cc = cc.yzx;
				ss = 5;
			}
		} else ss = 1;
	} else {
		if (pp.z >= 0) {
			if (pp.y >= 0) {
				cc = cc.yzx;
				ss = 2;
			} else {
				cc = cc.zxy;
				ss = 3;
			}
		}
	}
	float factor = 1-cc.y;
	int xInt = int(cc.x*texSize);
	int zInt = int(cc.z*texSize);
	float px = (xInt-zInt)/factor/texSize;
	vec2 tex = vec2(min(1 + px, 1), min(1 - px, 1)) * factor;
	int texOffset = fragIn.blockTex >> (16 + 3 * (5 - ss)) & 7;	//0111222333444555 + 16 bits
	color = texture(texSampler, vec3(tex, texDepth + texOffset));
#endif
	
	vec3 sunDir = normalize(sun);
	float visibility = max(min(dot(fragIn.normal, sunDir) * 1.2, 1.0), 0.7);// * (max(sunDir.y * 0.8, 0.0) + 0.2);
	
	color.rgb *= glow * visibility;
}