package hexagon

import org.lwjgl.glfw.GLFW._
import org.joml.Vector3f

class Player(val camera: Camera) {
  val velocity = new Vector3f
  private var flying = true
  var moveWithMouse = false
  
  camera.setPosition(0, 1.75f, 0)
//  camera.setPosition(0, 150, 0)
//  camera.setRotation((math.Pi/2).toFloat, 0, 0)
  
  def tick(): Unit = {
    updateCamera
  }
  // TODO: make Map[key: Int, state: Int] so that the game only receives key presses when it's not overlayed, or make this method not always be called
  private def updateCamera(): Unit = {
    val speed = if (glfwGetKey(Main.window, GLFW_KEY_LEFT_CONTROL) == GLFW_PRESS) 0.075f else if (glfwGetKey(Main.window, GLFW_KEY_RIGHT_CONTROL) == GLFW_PRESS) 120f else 3f
    val rSpeed = 0.05f
    if (flying) velocity.y = 0
    if (glfwGetKey(Main.window, GLFW_KEY_W) == GLFW_PRESS) {
			velocity.z -= (Math.cos(camera.rotation.y) * speed).toFloat
			velocity.x += (Math.sin(camera.rotation.y) * speed).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_S) == GLFW_PRESS) {
			velocity.z += (Math.cos(camera.rotation.y) * speed).toFloat
			velocity.x -= (Math.sin(camera.rotation.y) * speed).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_D) == GLFW_PRESS) {
			velocity.x += (Math.cos(camera.rotation.y) * speed).toFloat
			velocity.z += (Math.sin(camera.rotation.y) * speed).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_A) == GLFW_PRESS) {
			velocity.x -= (Math.cos(camera.rotation.y) * speed).toFloat
			velocity.z -= (Math.sin(camera.rotation.y) * speed).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_SPACE) == GLFW_PRESS) {
			if (flying) velocity.y = speed
			else if (velocity.y == 0) velocity.y = 7f
		}
		if (glfwGetKey(Main.window, GLFW_KEY_LEFT_SHIFT) == GLFW_PRESS) {
			if (flying) velocity.y = -speed
		}
		
		if (glfwGetKey(Main.window, GLFW_KEY_UP) == GLFW_PRESS) {
			camera.rotation.x -= rSpeed
			if (camera.rotation.x < -Math.PI / 2) camera.rotation.x = -(Math.PI / 2).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_DOWN) == GLFW_PRESS) {
			camera.rotation.x += rSpeed
			if (camera.rotation.x > Math.PI / 2) camera.rotation.x = (Math.PI / 2).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_LEFT) == GLFW_PRESS) {
			camera.rotation.y -= rSpeed
			if (camera.rotation.y < 0) camera.rotation.y += (Math.PI * 2).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_RIGHT) == GLFW_PRESS) {
			camera.rotation.y += rSpeed
			if (camera.rotation.y > Math.PI * 2) camera.rotation.y -= (Math.PI * 2).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_PAGE_UP) == GLFW_PRESS) {
			camera.rotation.z -= rSpeed
			if (camera.rotation.z < 0) camera.rotation.z += (Math.PI * 2).toFloat
		}
		if (glfwGetKey(Main.window, GLFW_KEY_PAGE_DOWN) == GLFW_PRESS) {
			camera.rotation.z += rSpeed
			if (camera.rotation.z > Math.PI * 2) camera.rotation.z -= (Math.PI * 2).toFloat
		}
		
		if (moveWithMouse) {
			val mouseMoved = Main.mouseMoved
			camera.rotation.y += mouseMoved.x.toFloat * rSpeed * 0.1f
			camera.rotation.x -= mouseMoved.y.toFloat * rSpeed * 0.1f
			
			if (camera.rotation.x < -Math.PI / 2) camera.rotation.x = -(Math.PI / 2).toFloat
			else if (camera.rotation.x > Math.PI / 2) camera.rotation.x = (Math.PI / 2).toFloat
			if (camera.rotation.y < 0) camera.rotation.y += (Math.PI * 2).toFloat
			else if (camera.rotation.y > Math.PI * 2) camera.rotation.y -= (Math.PI * 2).toFloat
		}
		
		velocity.mul(0.75f)
		
		val velLen = velocity.length()
		if (velLen > speed) velocity.mul(speed / velLen)
		
		camera.move(velocity.x / 60, velocity.y / 60, velocity.z / 60)
  }
}