package hexagon.resource

import java.io.File

import org.lwjgl.BufferUtils
import org.lwjgl.opengl.EXTTextureFilterAnisotropic
import org.lwjgl.opengl.GL
import org.lwjgl.opengl.GL11
import org.lwjgl.opengl.GL12
import org.lwjgl.opengl.GL30

import javax.imageio.ImageIO

object TextureArray {
  private val textures = collection.mutable.Map.empty[String, TextureArray]
  
  private var boundTextureArray: TextureArray = null
  
  def getTextureArray(name: String) = textures.get(name).get
  def registerTextureArray(name: String, texSize: Int): TextureArray = {
    if (!textures.contains(name)) textures.put(name, new TextureArray(name, texSize))
    textures(name)
  }
}

class TextureArray(val name: String, val texSize: Int) extends Resource {
  private var texID: Int = _
  
  TextureArray.textures += name -> this
  
  reload()
  
  def reload(): Unit = {
    val image = ImageIO.read(new File("res/" + name + ".png"))
    val width = image.getWidth
    val height = image.getHeight
    val pix = image.getRGB(0, 0, width, height, null, 0, width)
    val buf = BufferUtils.createByteBuffer(pix.length * 4)
    for (y <- 0 until width / texSize) {
      for (x <- 0 until height / texSize) {
        for (j <- 0 until texSize) {
          for (i <- 0 until texSize) {
            val idx = x * texSize + i + (y * texSize + j) * width
            buf.put((pix(idx) >> 16).toByte).put((pix(idx) >> 8).toByte).put((pix(idx) >> 0).toByte).put((pix(idx) >> 24).toByte)
          }
        }
      }
    }
    buf.flip
    texID = GL11.glGenTextures()
    GL11.glBindTexture(GL30.GL_TEXTURE_2D_ARRAY, texID)
    GL12.glTexImage3D(GL30.GL_TEXTURE_2D_ARRAY, 0, GL11.GL_RGBA, texSize, texSize, width / texSize * height / texSize, 0, GL11.GL_RGBA, GL11.GL_UNSIGNED_BYTE, buf)
    if (GL.getCapabilities().GL_EXT_texture_filter_anisotropic) {
			val amt = Math.min(16f, GL11.glGetFloat(EXTTextureFilterAnisotropic.GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT));
			// This does not work for the triangular images!
//			GL11.glTexParameterf(GL30.GL_TEXTURE_2D_ARRAY, EXTTextureFilterAnisotropic.GL_TEXTURE_MAX_ANISOTROPY_EXT, amt);
		} else {
			System.out.println("Anisotropic filtering is not supported :(");
		}
    GL11.glTexParameteri(GL30.GL_TEXTURE_2D_ARRAY, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST_MIPMAP_LINEAR)
		GL11.glTexParameteri(GL30.GL_TEXTURE_2D_ARRAY, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST)
		GL11.glTexParameteri(GL30.GL_TEXTURE_2D_ARRAY, GL11.GL_TEXTURE_WRAP_S, GL12.GL_CLAMP_TO_EDGE)
		GL11.glTexParameteri(GL30.GL_TEXTURE_2D_ARRAY, GL11.GL_TEXTURE_WRAP_T, GL12.GL_CLAMP_TO_EDGE)
		GL30.glGenerateMipmap(GL30.GL_TEXTURE_2D_ARRAY)
  }
  
  def bind(): Unit = {
    if (TextureArray.boundTextureArray != this) {
      TextureArray.boundTextureArray = this
      GL11.glBindTexture(GL30.GL_TEXTURE_2D_ARRAY, texID)
    }
  }
  
  def free(): Unit = {
    GL11.glDeleteTextures(texID)
    deleteResource()
  }
}