package hexagon.world.render

import org.lwjgl.opengl.GL11

import hexagon.Camera
import hexagon.renderer.NoDepthTest
import hexagon.renderer.Renderer
import hexagon.renderer.VAO
import hexagon.renderer.VAOBuilder
import hexagon.renderer.VBO
import hexagon.resource.Shader
import hexagon.world.storage.Block
import hexagon.world.storage.World
import org.lwjgl.BufferUtils
import org.joml.Vector3f
import hexagon.world.coord.BlockRelWorld
import hexagon.world.coord.CylCoord
import hexagon.renderer.InstancedRenderer

class WorldRenderer(world: World) {
	val blockShader = Shader.getShader("block")
	val blockSideShader = Shader.getShader("blockSide")
  val skyShader = Shader.getShader("sky")
  val selectedBlockShader = Shader.getShader("selectedBlock")
  
  private val skyVAO: VAO = new VAOBuilder(8).addVBO(VBO(4).floats(0, 2).create().fillFloats(0, Seq(-1, -1, 1, -1, -1, 1, 1, 1))).create()
  private val skyRenderer = new Renderer(skyVAO, GL11.GL_TRIANGLE_STRIP) with NoDepthTest
  
  private val selectedBlockVAO: VAO = new VAOBuilder(25)
      .addVBO(VBO(25).floats(0, 3).create().fillFloats(0, {
        val expandFn: CylCoord => Seq[Float] = v => Seq((v.x * 1.0025).toFloat, ((v.y - 0.25) * 1.0025 + 0.25).toFloat, (v.z * 1.0025).toFloat)
        val fn: Int => Seq[Float] = s => Block.getVertices(s+2).flatMap(expandFn)
        Seq(0, 2, 4).flatMap(fn) ++ expandFn(Block.vertices(0)) ++ Seq(1, 3, 5).flatMap(fn)
      }))
      .addVBO(VBO(1, divisor = 1).ints(1, 3).floats(2, 3).create()).create()
  private val selectedBlockRenderer = new InstancedRenderer(selectedBlockVAO, GL11.GL_LINE_STRIP)
	
	private var selectedBlockAndSide: Option[(BlockRelWorld, Option[Int])] = None
	def getSelectedBlock = selectedBlockAndSide.map(_._1)
	def getSelectedSide = selectedBlockAndSide.flatMap(_._2)
	def getSelectedBlockAndSide = selectedBlockAndSide
  
	def setSelectedBlockAndSide(blockAndSide: Option[(BlockRelWorld, Option[Int])]): Unit = {
	  if (selectedBlockAndSide != blockAndSide) {
	    selectedBlockAndSide = blockAndSide
	    
	    blockAndSide match {
	      case Some((coords, Some(_))) =>
        	val buf = BufferUtils.createByteBuffer(6*4).putInt(coords.x).putInt(coords.y).putInt(coords.z).putFloat(0).putFloat(0).putFloat(0)
        	buf.flip()
          selectedBlockVAO.vbos(1).fill(0, buf)
	      case _ =>
	    }
	  }
	}
  
  def render(camera: Camera): Unit = {
    skyShader.enable()
    skyRenderer.render()
    
    blockShader.enable()
    world.columns.values.filter(c => !world.columnsAtEdge.contains(c.coords)).foreach(_.chunks.values.foreach(c => {
      if (c.renderer != null) c.renderer.render_block()
    }))
    
    blockSideShader.enable()
    world.columns.values.filter(c => !world.columnsAtEdge.contains(c.coords)).foreach(_.chunks.values.foreach(c => {
      if (c.renderer != null) c.renderer.render_blockSide()
    }))
    
    if (getSelectedSide != None) {
      selectedBlockShader.enable()
      selectedBlockRenderer.render()
    }
  }
}