package hexagon.world.storage

import hexagon.world.coord.ColumnRelWorld
import hexagon.world.coord.ChunkRelColumn
import hexagon.world.coord.BlockRelColumn
import hexagon.Camera
import hexagon.world.coord.CylCoord
import org.joml.Vector2d
import hexagon.world.coord.CoordUtils
import hexagon.world.coord.ChunkRelWorld
import org.joml.Vector3d
import scala.concurrent.Future

object ChunkColumn {
  val neighbors = Seq(
      ( 1,  0),
      ( 0,  1),
      (-1,  0),
      ( 0, -1))
}

class ChunkColumn(val coords: ColumnRelWorld, val world: World) {
  /* chunks
   * method for loading/unloading top and bottom chunk
  */
  
  val chunks = scala.collection.mutable.Map.empty[Int, Chunk]
  private var topAndBottomChunks: Option[(Int, Int)] = None
  
  def getChunk(coords: ChunkRelColumn) = chunks.get(coords.value)
  
  def getBlock(coords: BlockRelColumn): Option[Block] = getChunk(coords.getChunkRelColumn) match {
    case Some(chunk) => chunk.getBlock(coords.getBlockRelChunk)
    case None => None
  }
  
  def tick(): Unit = {
    chunks.values.foreach(_.tick)
  }
  
  private[storage] def updateLoadedChunks(camera: Camera): Unit = {
    val origin = CylCoord(camera.position).toBlockCoord().toVector3d.div(16)
    
    def inSight(chunk: ChunkRelColumn): Boolean = {
  		math.abs(chunk.Y - origin.y) <= world.renderDistance
    }
    
    topAndBottomChunks match {
      case Some((top, bottom)) =>
        val bottomChunk = ChunkRelColumn(bottom & 0xfff)
        val newBottom = if (inSight(bottomChunk)) {
          val below = ChunkRelColumn((bottom - 1) & 0xfff)
          if (inSight(below)) {
            world.addChunkToLoadingQueue(below.withColumn(coords))
            bottom - 1
          } else bottom
        } else {
          chunks.remove(bottomChunk.value).foreach(_.unload)
          bottom + 1
        }
        
        val newTop = if (newBottom <= top) {
          val topChunk = ChunkRelColumn(top & 0xfff)
          
          if (inSight(topChunk)) {
            val above = ChunkRelColumn((top + 1) & 0xfff)
            if (inSight(above)) {
              world.addChunkToLoadingQueue(above.withColumn(coords))
              top + 1
            } else top
          } else {
            chunks.remove(topChunk.value).foreach(_.unload)
            top - 1
          }
        } else top
        
        topAndBottomChunks = if (newBottom <= newTop) Some((newTop, newBottom)) else None
      case None =>
        val first = ChunkRelColumn(math.round(origin.y).toInt & 0xfff)
        if (inSight(first)) {
          world.addChunkToLoadingQueue(first.withColumn(coords))
          topAndBottomChunks = Some((first.Y, first.Y))
        }
    }
  }
  
  def unload(): Unit = {
    chunks.values.foreach(_.unload)
  }
  
}