package hexagon.world.blockType

import hexagon.resource.TextureArray

object BlockType {
  private val blocks = collection.mutable.Map.empty[Byte, BlockType]
  private val blockTextures = collection.mutable.ArrayBuffer.empty[Array[Int]]
  def byId(id: Byte): BlockType = blocks(id)
  
  val Air       = BlockTypeAir
  val Grass     = new BlockType(1, "grass", "Grass", s => if (s == 0) 1 else if (s == 1) 2 else 0)
  val Dirt      = new BlockType(2, "dirt", "Dirt")
  val Stone     = new BlockType(3, "stone", "Stone")
  val Sand      = new BlockType(4, "sand", "Sand")

  def init(): Unit = {
    TextureArray.registerTextureArray("blocks", BlockTexture.blockTextureSize, blockTextures)
  }
}

class BlockType(val id: Byte, val name: String, val displayName: String, texIdxFunc: Int => Int = _ => 0) {
  BlockType.blocks(id) = this
  
  protected val texIndices: Seq[Int] = for (s <- 0 until 8) yield texIdxFunc(s)
  protected val texture = new BlockTexture("res/textures/blocks/" + name + ".png")
  protected val indexInTextureArray = BlockType.blockTextures.size
  BlockType.blockTextures ++= texture.images
  
  def blockTex(side: Int): Int = (indexInTextureArray + texIndices(side))
  def canBeRendered: Boolean = true
}

object BlockTypeAir extends BlockType(0, "air", "Air") {
  override def canBeRendered: Boolean = false
}
