package hexagon

import hexagon.world.coord.CylCoord
import hexagon.world.storage.World
import hexagon.world.coord.CoordUtils
import hexagon.world.coord.BlockCoord
import hexagon.world.coord.BlockRelWorld
import hexagon.world.coord.SkewCylCoord
import org.joml.Vector3d

/** radius is the big radius of the hexagon */
class HexBox(val radius: Float, val bottom: Float, val top: Float) {
  val smallRadius = radius * CoordUtils.y60 / 2
  
  /** pos and velocity should be CylCoords in vector form */
  def collides(pos: Vector3d, velocity: Vector3d, timeStep: Double, world: World): Boolean = {
    val skewCoord = CylCoord(pos.x + velocity.x * timeStep, pos.y + velocity.y * timeStep, pos.z + velocity.z * timeStep).toSkewCylCoord()
    val blockCoord = skewCoord.toBlockCoord()
    var free = true
    for (y <- math.floor((pos.y + velocity.y * timeStep + bottom) * 2).toInt to math.floor((pos.y + velocity.y * timeStep + top) * 2).toInt) {
      for (x <- -1 to 1) {
        for (z <- -1 to 1) {
          if (x * z != 1) {// corners
            val coords = BlockRelWorld(blockCoord.x.toInt + x, y, blockCoord.z.toInt + z)
            world.getBlock(coords).map(_.blockType) match {
              case Some(blockType) =>
                if (intersect(skewCoord, blockType.bounds, BlockCoord(coords.x, coords.y, coords.z).toSkewCylCoord())) {// more complicated
                  if (intersect(skewCoord, blockType.bounds, BlockCoord(coords.x, coords.y, coords.z).toSkewCylCoord()) != blockType.bounds.intersect(BlockCoord(coords.x, coords.y, coords.z).toSkewCylCoord(), this, skewCoord)) {
                    println("Oops! We have a problem!")
                  }
                  free = false
                }
              case None =>
            }
          }
        }
      }
    }
    println(!free)
    !free
  }
  
  private def intersect(position: SkewCylCoord, otherBox: HexBox, otherPos: SkewCylCoord): Boolean = {
    val intY = otherPos.y + otherBox.top > position.y + bottom && otherPos.y + otherBox.bottom < position.y + top
    val intX = otherPos.x + otherBox.smallRadius > position.x - smallRadius && otherPos.x - otherBox.smallRadius < position.x + smallRadius
    val intZ = otherPos.z + otherBox.smallRadius > position.z - smallRadius && otherPos.z - otherBox.smallRadius < position.z + smallRadius
    val intW = otherPos.z - otherPos.x + otherBox.smallRadius > position.z - position.x - smallRadius && otherPos.z - otherPos.x - otherBox.smallRadius < position.z - position.x + smallRadius
//    println(s"${otherPos.y + otherBox.top} > ${position.y + bottom} && ${otherPos.y + otherBox.bottom} < ${position.y + top}")
/*    if (intY) println("y:\t" + ((otherPos.y + otherBox.top) - (position.y + bottom)) + "\t" + -((otherPos.y + otherBox.bottom) - (position.y + top)))
    if (intY) println("x:\t" + ((otherPos.x + otherBox.smallRadius) - (position.x - smallRadius)) + "\t" + -((otherPos.x - otherBox.smallRadius) - (position.x + smallRadius)))
    if (intY) println("z:\t" + ((otherPos.z + otherBox.smallRadius) - (position.z - smallRadius)) + "\t" + -((otherPos.z - otherBox.smallRadius) - (position.z + smallRadius)))
    if (intY) println("w")
    if (intY || intX || intZ || intW) println()*/
    
    intY && intX && intZ// && intW
  }
}
